from __future__ import annotations

__doc__ = """
This package contains the main entry point and command structure for the
optpricing command-line interface.
"""

from .main import app

__all__ = ["app"]
