from __future__ import annotations

__doc__ = """
This package contains the individual command implementations for the CLI.

These modules are imported by the main CLI entry point but are not considered
part of the public API of the library.
"""
